'''
Controller for the K74V2 parallel port relay board.
Great relay board, but inconvenient in that it requires
setting all the relays at once, which means you have to
keep track of what's been set. So this program writes
to a text file (status.txt) in the same directory to
help it remember.

Note that if you're using it in a script that runs persistently,
you don't need to write the status to a text file and you can
ditch the update_history_file() function.

Relay board:
http://www.electronickits.com/kit/complete/elec/ck1601.htm

Requires installing GiveIO, which was included. Note that
it might be a security risk. See:
http://linuxgazette.net/112/radcliffe.html

Might also require simpleio.dll to be placed in the windows/system32 folder.

And of course requires the pyparallel module.

Feel free to email wrybread@gmail.com if you can't get it working.

Usage:
k74 1 0 turns relay 1 off.
k74 2 1 turns relay 2 on
k74 0 1 turns all relays on
k74 0 0 turns all relays off

Can also use "on/off". For example:
k74 1 on turns relay 1 on
'''




import parallel
import sys, os


def update_history_file(relay, state):

    '''
    This reads the previous relay state from a pickle file. 
    You don't need this unless you want to keep track of the
    relay states between program runs.
    '''

    # figure out the current directory in such a way that doesn't throw off pyinstaller singlefile
    current_directory_raw = os.path.abspath(sys.argv[0])
    current_directory_raw = os.path.split(current_directory_raw)
    current_directory = current_directory_raw[0] + "\\"

    # File that'll store our pickled dictionary of previous states.
    status_file = current_directory + "k74_status.txt"

    # Load our pickled dictionary
    import pickle
    try:
        filehandler = open(status_file, 'r')
        relays_dict = pickle.load(filehandler)
    except IOError:
        # Doesn't exist, create a blank one
        relays_dict = {}
        relays_dict = {1:0, 2:0, 3:0, 4:0, 5:0, 6:0, 7:0, 8:0}

    # Update it with our new relay state
    relays_dict[relay] = state

    # If we're turning all relays on or off...
    if relay == 0 and state == 0: relays_dict = {1:0, 2:0, 3:0, 4:0, 5:0, 6:0, 7:0, 8:0}   # all off
    elif relay == 0 and state == 1: relays_dict = {1:1, 2:1, 3:1, 4:1, 5:1, 6:1, 7:1, 8:1} # all on

    # Save the status to our pickle file
    filehandler = open(status_file, 'w')
    pickle.dump(relays_dict, filehandler)
    filehandler.close()

    return relays_dict



def k74(relay, state):

    # Convert things to binary
    if state.lower() == "off": state = 0
    if state.lower() == "on": state = 1

    if relay.lower() == "all": relay = 0

    # Make sure we have integers
    relay = int(relay)
    state = int(state)

    if state > 1: state = 1

    global relays_dict # made global to keep track of relay states in scripts that run persistently

    # Read the previous status from a text file. Comment this out if you're
    # keeping your program running.
    relays_dict = update_history_file(relay, state)

    # Check if the relay dictionary has been defined yet
    try: relays_dict
    except NameError:
        # Define it
        relays_dict = {}
        for i in range(1, 8): relays_dict[i] = 0

    # Now set the new relay value in our dictionary
    relays_dict[relay] = state

    # Build the output string (there's probably a better way to do this)
    output = ""
    for key in relays_dict.iterkeys():
        output += str( relays_dict[key] )

    # Reverse it
    output = output[::-1]

    # Handle all on and all off, as in k74(0, 1) for all on
    if relay == 0 and state == 0: output = "00000000"
    if relay == 0 and state == 1: output = "11111111"

    # Convert it to binary:
    x = int(output, 2)

    # Write to the parallel port:
    import parallel
    port = parallel.Parallel()
    port.setData(x)




k74( sys.argv[1], sys.argv[2] )









